__doc__ = """
ZUnit tests for the DataObject class
"""

from Products import ZUnit
import DataObject, utils
import string

class creationTest (ZUnit.TestCase):
	def runTest (self):
		"Try to create a DataObject"
		ctx = self.ZopeContext
		ctx.REQUEST.set ('title', 'Testing creation')
		ctx.REQUEST.set ('template', 'None')
		DataObject.manage_addDataObject (ctx, 'created', ctx.REQUEST)
		self.failUnless ('created' in ctx.objectIds(), 'Couldn\'t create DataObject')
		try:
			template = ctx.created.template
			self.failUnless (template == 'None', 'DataObject not properly initialized')
		except:
			self.fail ('DataObject not properly initialized, couldn\'t retrieve template:' + ZUnit.SubTraceback())

class objectTests (ZUnit.TestCase):
	def setUp (self):
		ctx = self.ZopeContext
		self.r = ctx.REQUEST
		self.r.set ('title', 'Testing DataObject')
		DataObject.manage_addDataObject (ctx, 'dataobj', ctx.REQUEST)
		self.o = ctx.dataobj

	def tearDown (self):
		self.ZopeContext.manage_renameObject ('dataobj',
			'dataobj%02d' % len (self.ZopeContext.objectIds()))

	def test01_manage_propertiesForm (self):
		"Try to access manage_propertiesForm"
		try:
			page = self.o.manage_propertiesForm (self.o, self.r)
		except:
			self.fail ('Couldn\'t view management screen (manage_propertiesForm):' + ZUnit.subTraceback())
		self.failUnless (string.find (page, '<input type="text" name="template:string" size="35"') > 0,
			'Management screen corrupted (manage_propertiesForm)')
	
	def test02_manage_addProperty (self):
		"Try to add new properties to the object"
		for prop, val, ptype in (
			('foo', 0, 'int'), ('bar', '', 'string'), ('bar_filter', '', 'string')):
			try:
				self.o.manage_addProperty (prop, val, ptype)
			except:
				self.fail ('Couldn\'t add property %s: %s' % (prop, ZUnit.subTraceback()))
			self.failUnless (hasattr (self.o, prop), 'Property %s not added!' % prop)

	def test03_manage_editProperties (self):
		"Try to set some properties, including the template and a filter"
		self.test02_manage_addProperty()
		try:
			self.o.manage_editProperties ({
				'title': 'this is a test',
				'template': 'test_renderer',
				'foo': 5,
				'bar': 'Madame Satan',
				'bar_filter': 'test_filter',
			})
		except:
			self.fail ('Couldn\'t edit properties:' + ZUnit.subTraceback())
		self.failUnless (self.o.title == 'this is a test' and
			self.o.template == 'test_renderer' and self.o.foo == 5 and
			self.o.bar == 'Madame Satan' and self.o.bar_filter == 'test_filter',
			'Properties not correctly edited!')
	
	def test04_manage_delProperties (self):
		"Try to delete properties"
		self.test03_manage_editProperties()
		try:
			self.o.manage_delProperties (('bar_filter', 'foo'))
		except:
			self.fail ('Couldn\'t delete properties:' + ZUnit.subTraceback())
		self.failIf (hasattr (self.o, 'foo') or hasattr (self.o, 'bar_filter'),
			'Properties not deleted!')
		self.failUnless (hasattr (self.o, 'bar'), 'Too many properties deleted!')
	
	def makeRenderTest (self, substring, invert=None):
		"Try to render the object, then check if the results contain 'substring'"
		try:
			p = str (self.o)
		except:
			self.fail ('Couldn\'t render object:' + ZUnit.subTraceback())
		if invert:
			test = self.failIf
		else:
			test = self.failUnless
		test (string.find (p, substring) >= 0,
			'The object was rendered incorrectly:\n		 ' +
			string.join (string.split (p, '\n'), '\n		 '))
	
	def test05_renderSimpleTemplate (self):
		"Try to render an object with a very simplistic template"
		self.test04_manage_delProperties()
		def renderer (ctx, req, **kw):
			return 'success'
		self.o.test_renderer = renderer
		self.makeRenderTest ('success')
	
	def test06_renderTemplateWithProperties (self):
		"Try to render an object with a template that accesses properties"
		self.test04_manage_delProperties()
		def renderer (ctx, req, **kw):
			return utils.evaluate ('bar', ctx, req, kw)
		self.o.test_renderer = renderer
		self.makeRenderTest ('Madame Satan')
	
	def test07_renderTemplateWithFilters (self):
		"Try to render an object with a template that accesses filtered properties"
		self.test03_manage_editProperties()
		def renderer (ctx, req, **kw):
			return utils.evaluate ('bar', ctx, req, kw)
		self.o.test_renderer = renderer
		self.o.test_filter = string.lower
		self.makeRenderTest ('madame satan')

def creator():
	suite = ZUnit.TestSuite()
	suite.addTest (creationTest())
	suite.addTest (ZUnit.makeSuite (objectTests))
	return suite
